# Some simple examples using libiio and libiio-cpp

## Dependencies

This project relies on my [libiio C++ bindings](https://gitlab.ost.ch/moritz.lammerich/libiio-cpp).

## Examples

### device-info

`device-info` creates a local context and goes through each available device, printing its attributes.
It also checks each device for available channels and goes through them, listing their attributes as well.

### read-adc

`read-adc` was created specifically for the Beaglebone Blue and expects the `TI-am335x-adc.0.auto` device to be present on the system.
If the ADC is found, it creates a buffer and reads one sample, outputting it to the console.

### set-trigger

`set-trigger` sets up a high resolition timer as a trigger for the dummy device. Note the `iio_dummy` and `iio_trig_hrtimer` kernel modules must be loaded (see setup below).

#### Setup

first, load the necessary kernel modules:
```
modprobe iio_dummy
modprobe iio_trig_hrtimer
```

Now, create a new high resolution timer called `my_periodic`. `set-trigger` will look for this name, so it has to be `my_periodic`.
```
mkdir /sys/kernel/config/iio/triggers/hrtimer/my_periodic
```

`set-trigger` will take care of the configuration, namely setting an appropriate sampling frequency.
Now, create a dummy device called `foo`. Again, this name is used by `set-trigger` to find the device, so it has to be `foo`.

```
mkdir /sys/kernel/config/iio/devices/dummy/foo
```

As a sanity check, you can `/sys/bus/iio/devices` to make sure the devices show up there

```
root@bblue:~# ls /sys/bus/iio/devices/
iio:device0  iio:device1  iio_evgen  trigger0
```

Now you can run `set-trigger`.

## Troubleshooting

Libiio errors are turned into exceptions by `libiio-cpp`, so any errors should be printged to the console before the program terminates.

If a previous execution of a program failed, you may encounter a `device or resource busy` error when trying to create a buffer.
This is because the buffer was not disabled (probably due to an exception in the previous run) and can be fixed by manually disabling the buffer:

```
echo 0 > /sys/bus/iio/devices/iio\:device1/buffer/enable
```
