#include <iostream>

#include "libiio-cpp.hpp"

int main() {
  libiio::Context context;
  auto deviceCount = context.deviceCount();
  std::cout << "found " << deviceCount << " devices\n";
  for (auto i = 0; i < deviceCount; ++i) {
    std::cout << "enumerating " << i << '\n';
    auto device = context.deviceAtIndex(i);
    auto channelCount = device.channelCount();
    std::cout << "\tdevice " << i << " " << device.id() << " => "
              << device.name() << " with " << channelCount << " channels and "
              << device.attributeCount() << " attributes\n";
    if (device.isTrigger())
      std::cout << "\tdevice is a trigger\n";
    else
      std::cout << "\tdevice is not a trigger\n";

    // somehow iterating over all device attributes doesn't work, so do it
    // manually
    for (auto i = 0; i < device.attributeCount(); ++i) {
      std::cout << "\t\t\t attribute " << i << ": "
                << device.attributeAtIndex(i) << "\n";
    }
    for (auto i = 0; i < channelCount; ++i) {
      auto channel = device.channelAtIndex(i);
      std::cout << "\t\tchannel " << i << " " << channel.id() << " => "
                << channel.name();
      if (channel.isOutput())
        std::cout << " (output) ";
      else
        std::cout << " (input)  ";
      std::cout << channel.attributeCount() << " attributes\n";
      channel.iterateOverAttributes(
          [](auto channel, auto name, const char *data, std::size_t size) {
            std::cout << "\t\t\t attribute: " << name << " size " << size
                      << " value: ";
            for (auto i = 0; i < size; ++i)
              std::cout << (data[i]);
            std::cout << "\n";
          });
    }
  }
  std::cout << "done\n";
}