#include <chrono>
#include <iostream>
#include <thread>

#include "libiio-cpp.hpp"

using namespace libiio;

void printTrigger(Device device) {

  auto trigger = device.getTrigger();
  if (trigger) {
    std::cout << "trigger is set to " << trigger->name() << "\n";
  } else {
    std::cout << "trigger is not set\n";
  }
}

void printDataAvailable(Device device) {
  std::cout << "data available?: "
            << (device.readBufferAttribute("data_available") ? "yes" : "no")
            << "\n";
}

int main() {
  Context context;
  auto trigger = context.findDevice("my_periodic");
  std::cout << "found trigger!\n";
  auto device = context.findDevice("foo");
  std::cout << "found device\n";

  printTrigger(device);

  trigger.setAttribute("sampling_frequency", 100);
  auto timestamp = device.findChannel("timestamp");
  timestamp.enable();
  auto channel = device.findChannel("voltage0");
  if (!channel.isScanElement()) {
    std::cout << "channel is not a scan element and can not be generate any "
                 "samples\n";
    return 1;
  } else {
    std::cout << "channel is scan element\n";
  }
  channel.enable();
  device.setTrigger(trigger);
  printTrigger(device);
  auto buffer = device.createBuffer(32);
  printDataAvailable(device);
  std::this_thread::sleep_for(std::chrono::seconds(1));
  printDataAvailable(device);
  std::cout << "refilling buffer\n";
  auto n = buffer.refill();
  std::cout << "refilled " << n << " bytes\n";
  printDataAvailable(device);
  auto format = channel.getDataFormat();
  std::cout << "data format: " << format.repeat() << " samples of "
            << format.lengthBytes() << " bytes\n";
  auto samples = channel.read<std::uint16_t>(buffer);
  std::cout << "data: ";
  for (auto sample : samples) {
    std::cout << sample << " ";
  }
  std::cout << "\ndone\n";
}